@@ -59,6 +59,8 @@ class ML_Lag_Regimes(BaseML_Lag, REGI.Regimes_Frame):
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
                    If slx_lags>0, the specification becomes of the Spatial Durbin type.                   
+    slx_vars     : either "all" (default) or list of booleans to select x variables
+                   to be lagged       
     regime_lag_sep: boolean
                     If True, the spatial parameter for spatial lag is also
                     computed according to different regimes. If False (default),
@@ -320,6 +322,7 @@ class ML_Lag_Regimes(BaseML_Lag, REGI.Regimes_Frame):
         method="full",
         epsilon=0.0000001,
         slx_lags=0,
+        slx_vars="all",
         regime_lag_sep=False,
         cores=False,
         spat_diag=True,
@@ -344,10 +347,11 @@ class ML_Lag_Regimes(BaseML_Lag, REGI.Regimes_Frame):
         name_x = USER.set_name_x(name_x, x_constant, constant=True)
 
         if slx_lags > 0:
-            lag_x = get_lags(w, x_constant, slx_lags)
-            x_constant = np.hstack((x_constant, lag_x))
-            name_x += USER.set_name_spatial_lags(name_x, slx_lags)
-            kwx = lag_x.shape[1]
+            size_x = x_constant.shape[1]
+            x_constant,name_x = USER.flex_wx(w,x=x_constant,name_x=name_x,constant=False,
+                                                slx_lags=slx_lags,slx_vars=slx_vars)
+            set_warn(self,"WX is computed using the complete W, i.e. not trimmed by regimes.")
+            kwx = x_constant.shape[1] - size_x
 
         self.name_x_r = USER.set_name_x(name_x, x_constant) + [USER.set_name_yend_sp(name_y)]
         self.method = method
@@ -414,7 +418,7 @@ class ML_Lag_Regimes(BaseML_Lag, REGI.Regimes_Frame):
         else:
             # if regime_lag_sep == True:
             #    w = REGI.w_regimes_union(w, w_i, self.regimes_set)
-            x, self.name_x, x_rlist = REGI.Regimes_Frame.__init__(
+            x, self.name_x, xtype, x_rlist = REGI.Regimes_Frame.__init__(
                 self,
                 x_constant,
                 regimes,
@@ -434,7 +438,8 @@ class ML_Lag_Regimes(BaseML_Lag, REGI.Regimes_Frame):
             self.regime_lag_sep = regime_lag_sep
             self.output = pd.DataFrame(self.name_x, columns=['var_names'])
             self.output['regime'] = x_rlist + ['_Global']
-            self.output['var_type'] = ['x'] * (len(self.name_x) - 1) + ['rho']
+            self.output['var_type'] = xtype + ['rho']
+            #self.output['var_type'] = ['o'] + ['x'] * (len(self.name_x) - 2) + ['rho']
             self.output['equation'] = 0
             self.slx_lags = slx_lags
             diag_out = None
@@ -444,7 +449,7 @@ class ML_Lag_Regimes(BaseML_Lag, REGI.Regimes_Frame):
                 kwx = kwx - fixed_wx
                 if kwx > 0:
                     for m in self.regimes_set:
-                        r_output = self.output[(self.output['regime'] == str(m)) & (self.output['var_type'] == 'x')]
+                        r_output = self.output[(self.output['regime'] == str(m)) & (self.output['var_type'].isin(['x', 'o']))]
                         wx_index = r_output.index[-kwx:]
                         self.output.loc[wx_index, 'var_type'] = 'wx'
                 if fixed_wx > 0:
@@ -602,9 +607,9 @@ class ML_Lag_Regimes(BaseML_Lag, REGI.Regimes_Frame):
             results[r].other_mid = ""
             if slx_lags > 0:
                 kx = (len(results[r].name_x) - 1) // (slx_lags + 1)
-                var_types = ['x'] * (kx + 1) + ['wx'] * kx * slx_lags + ['rho']
+                var_types = ['o']+['x']*kx + ['wx'] * kx * slx_lags + ['rho']
             else:
-                var_types = ['x'] * (len(results[r].name_x) - 1) + ['rho']
+                var_types = ['o']+['x'] * (len(results[r].name_x) - 2) + ['rho']
             results[r].output = pd.DataFrame({'var_names': results[r].name_x,
                                               'var_type': var_types,
                                               'regime': r, 'equation': r})
