@@ -4,9 +4,9 @@ __author__ = "Luc Anselin lanselin@gmail.com, Pedro Amaral pedrovma@gmail.com, D
 import numpy as np
 import numpy.linalg as la
 from . import user_output as USER
-from .output import output, _spat_diag_out, _nonspat_mid, _nonspat_top
+from .output import output, _spat_diag_out, _nonspat_mid, _nonspat_top, _summary_vif
 from . import robust as ROBUST
-from .utils import spdot, RegressionPropsY, RegressionPropsVM, set_warn, get_lags
+from .utils import spdot, RegressionPropsY, RegressionPropsVM, set_warn
 import pandas as pd
 
 __all__ = ["OLS"]
@@ -128,9 +128,9 @@ class OLS(BaseOLS):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
     w            : pysal W object
@@ -147,6 +147,12 @@ class OLS(BaseOLS):
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
                    If slx_lags>0, the specification becomes of the SLX type.
+    slx_vars     : either "All" (default) or list of booleans to select x variables
+                   to be lagged
+    regimes      : list or pandas.Series
+                   List of n values with the mapping of each
+                   observation to a regime. Assumed to be aligned with 'x'.
+                   For other regimes-specific arguments, see OLS_Regimes
     sig2n_k      : boolean
                    If True, then use n-k to estimate sigma^2. If False, use n.
     nonspat_diag : boolean
@@ -161,6 +167,8 @@ class OLS(BaseOLS):
     white_test   : boolean
                    If True, compute White's specification robust test.
                    (requires nonspat_diag=True)
+    vif          : boolean
+                   If True, compute variance inflation factor.
     vm           : boolean
                    If True, include variance-covariance matrix in summary
                    results
@@ -434,11 +442,14 @@ class OLS(BaseOLS):
         robust=None,
         gwk=None,
         slx_lags = 0,
+        slx_vars = "All",
+        regimes = None,
         sig2n_k=True,
         nonspat_diag=True,
         spat_diag=False,
         moran=False,
         white_test=False,
+        vif=False,
         vm=False,
         name_y=None,
         name_x=None,
@@ -446,56 +457,86 @@ class OLS(BaseOLS):
         name_gwk=None,
         name_ds=None,
         latex=False,
+        **kwargs,
     ):
-
-        n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
-        USER.check_robust(robust, gwk)
-        if robust == "hac" and spat_diag:
-                set_warn(
-                    self,
-                    "Spatial diagnostics are not available for HAC estimation. Hence, spatial diagnostics have been disabled for this model.",
-                )
-                spat_diag = False
-        if robust in ["hac", "white"] and white_test:
-                set_warn(
-                    self,
-                    "White test not available when standard errors are estimated by HAC or White correction.",
-                )
-                white_test = False
-        USER.check_spat_diag(spat_diag, w)
-        x_constant, name_x, warn = USER.check_constant(x, name_x)
-        self.name_x = USER.set_name_x(name_x, x_constant)
-        if slx_lags >0:
-            USER.check_weights(w, y, w_required=True)
-            lag_x = get_lags(w, x_constant[:, 1:], slx_lags)
-            x_constant = np.hstack((x_constant, lag_x))
-            self.name_x += USER.set_name_spatial_lags(self.name_x[1:], slx_lags)
+        if regimes is not None:
+            from .ols_regimes import OLS_Regimes
+            self.__class__ = OLS_Regimes
+            self.__init__(
+                y=y,
+                x=x,
+                regimes=regimes,
+                w=w,
+                robust=robust,
+                gwk=gwk,
+                slx_lags=slx_lags,
+                sig2n_k=sig2n_k,
+                nonspat_diag=nonspat_diag,
+                spat_diag=spat_diag,
+                moran=moran,
+                white_test=white_test,
+                vm=vm,
+                name_y=name_y,
+                name_x=name_x,
+                name_w=name_w,
+                name_gwk=name_gwk,
+                name_ds=name_ds,
+                latex=latex,
+                **kwargs,
+            )
         else:
-            USER.check_weights(w, y, w_required=False)
-        set_warn(self, warn)
-        BaseOLS.__init__(
-            self, y=y, x=x_constant, robust=robust, gwk=gwk, sig2n_k=sig2n_k
-        )
-        self.name_ds = USER.set_name_ds(name_ds)
-        self.name_y = USER.set_name_y(name_y)
-        self.slx_lags = slx_lags
-        self.title = "ORDINARY LEAST SQUARES"
-        if slx_lags > 0:
-            self.title += " WITH SPATIALLY LAGGED X (SLX)"
-        self.robust = USER.set_robust(robust)
-        self.name_w = USER.set_name_w(name_w, w)
-        self.name_gwk = USER.set_name_w(name_gwk, gwk)
-        self.output = pd.DataFrame(self.name_x, columns=['var_names'])
-        self.output['var_type'] = ['x'] * len(self.name_x)
-        self.output['regime'], self.output['equation'] = (0, 0)
-        self.other_top, self.other_mid, other_end = ("", "", "")  # strings where function-specific diag. are stored
-        if nonspat_diag:
-            self.other_mid += _nonspat_mid(self, white_test=white_test)
-            self.other_top += _nonspat_top(self)
-        if spat_diag:
-            other_end += _spat_diag_out(self, w, 'ols', moran=moran)
-        output(reg=self, vm=vm, robust=robust, other_end=other_end, latex=latex)
+            n = USER.check_arrays(y, x)
+            y, name_y = USER.check_y(y, n, name_y)
+            USER.check_robust(robust, gwk)
+            spat_diag, warn = USER.check_spat_diag(spat_diag=spat_diag, w=w, robust=robust, slx_lags=slx_lags)
+            set_warn(self, warn)
+
+            if robust in ["hac", "white"] and white_test:
+                    set_warn(
+                        self,
+                        "White test not available when standard errors are estimated by HAC or White correction.",
+                    )
+                    white_test = False
+
+            x_constant, name_x, warn = USER.check_constant(x, name_x)
+            set_warn(self, warn)
+            self.name_x = USER.set_name_x(name_x, x_constant)
+            
+            if spat_diag or moran:
+                w = USER.check_weights(w, y, slx_lags=slx_lags, w_required=True, allow_wk=True)
+            else:
+                w = USER.check_weights(w, y, slx_lags=slx_lags, allow_wk=True)
+            if slx_lags >0:
+    #            lag_x = get_lags(w, x_constant[:, 1:], slx_lags)
+    #            x_constant = np.hstack((x_constant, lag_x))
+    #           self.name_x += USER.set_name_spatial_lags(self.name_x[1:], slx_lags)
+                x_constant,self.name_x = USER.flex_wx(w,x=x_constant,name_x=self.name_x,constant=True,
+                                                slx_lags=slx_lags,slx_vars=slx_vars)
+
+            BaseOLS.__init__(
+                self, y=y, x=x_constant, robust=robust, gwk=gwk, sig2n_k=sig2n_k
+            )
+            self.name_ds = USER.set_name_ds(name_ds)
+            self.name_y = USER.set_name_y(name_y)
+            self.slx_lags = slx_lags
+            self.title = "ORDINARY LEAST SQUARES"
+            if slx_lags > 0:
+                self.title += " WITH SPATIALLY LAGGED X (SLX)"
+            self.robust = USER.set_robust(robust)
+            self.name_w = USER.set_name_w(name_w, w)
+            self.name_gwk = USER.set_name_w(name_gwk, gwk)
+            self.output = pd.DataFrame(self.name_x, columns=['var_names'])
+            self.output['var_type'] = ['o']+['x'] * (len(self.name_x)-1)
+            self.output['regime'], self.output['equation'] = (0, 0)
+            self.other_top, self.other_mid, other_end = ("", "", "")  # strings where function-specific diag. are stored
+            if nonspat_diag:
+                self.other_mid += _nonspat_mid(self, white_test=white_test)
+                self.other_top += _nonspat_top(self)
+            if vif:
+                self.other_mid += _summary_vif(self)
+            if spat_diag:
+                other_end += _spat_diag_out(self, w, 'ols', moran=moran)
+            output(reg=self, vm=vm, robust=robust, other_end=other_end, latex=latex)
 
 
 
