@@ -3,10 +3,11 @@ import multiprocessing as mp
 import pandas as pd
 from . import regimes as REGI
 from . import user_output as USER
-from .utils import set_warn, RegressionProps_basic, spdot, sphstack, get_lags
+from .utils import set_warn, RegressionProps_basic, spdot, sphstack, get_lags, optim_k
 from .twosls import BaseTSLS
 from .robust import hac_multi
 from .output import output, _spat_diag_out
+from .skater_reg import Skater_reg
 
 """
 Two-stage Least Squares estimation with regimes.
@@ -22,19 +23,19 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    yend         : array
+    yend         : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    endogenous variable
-    q            : array
+    q            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    external exogenous variable to use as instruments (note:
                    this should not contain any variables from x)
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     constant_regi: string
@@ -68,6 +69,8 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
                    Number of spatial lags of X to include in the model specification.
                    If slx_lags>0, the specification becomes of the SLX type.
                    Note: WX is computed using the complete weights matrix
+    slx_vars     : either "all" (default) or list of booleans to select x variables
+                   to be lagged                         
     sig2n_k      : boolean
                    If True, then use n-k to estimate sigma^2. If False, use n.
     vm           : boolean
@@ -193,61 +196,47 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
     >>> import libpysal
     >>> from libpysal.examples import load_example
     >>> from libpysal.weights import Rook
+    >>> import geopandas as gpd
 
-    Open data on NCOVR US County Homicides (3085 areas) using libpysal.io.open().
-    This is the DBF associated with the NAT shapefile.  Note that
-    libpysal.io.open() also reads data in CSV format; since the actual class
-    requires data to be passed in as numpy arrays, the user can read their
-    data in using any method.
+    Open data on NCOVR US County Homicides (3085 areas) using geopandas.
 
     >>> nat = load_example('Natregimes')
-    >>> db = libpysal.io.open(nat.get_path('natregimes.dbf'), 'r')
+    >>> db = gpd.read_file(nat.get_path('natregimes.shp'))
 
-    Extract the HR90 column (homicide rates in 1990) from the DBF file and make it the
-    dependent variable for the regression. Note that PySAL requires this to be
-    an numpy array of shape (n, 1) as opposed to the also common shape of (n, )
-    that other packages accept.
+    Select the HR90 column (homicide rates in 1990) as the
+    dependent variable for the regression. 
 
-    >>> y_var = 'HR90'
-    >>> y = np.array([db.by_col(y_var)]).reshape(3085,1)
+    >>> y = db['HR90']
 
-    Extract UE90 (unemployment rate) and PS90 (population structure) vectors from
-    the DBF to be used as independent variables in the regression. Other variables
-    can be inserted by adding their names to x_var, such as x_var = ['Var1','Var2','...]
-    Note that PySAL requires this to be an nxj numpy array, where j is the
-    number of independent variables (not including a constant). By default
-    this model adds a vector of ones to the independent variables passed in.
+    Select UE90 (unemployment rate) and PS90 (population structure) variables
+    as independent variables in the regression.
 
-    >>> x_var = ['PS90','UE90']
-    >>> x = np.array([db.by_col(name) for name in x_var]).T
+    >>> x = db[['PS90','UE90']]
 
     In this case we consider RD90 (resource deprivation) as an endogenous regressor.
     We tell the model that this is so by passing it in a different parameter
     from the exogenous variables (x).
 
-    >>> yd_var = ['RD90']
-    >>> yd = np.array([db.by_col(name) for name in yd_var]).T
+    >>> yd = db['RD90']
 
     Because we have endogenous variables, to obtain a correct estimate of the
     model, we need to instrument for RD90. We use FP89 (families below poverty)
     for this and hence put it in the instruments parameter, 'q'.
 
-    >>> q_var = ['FP89']
-    >>> q = np.array([db.by_col(name) for name in q_var]).T
+    >>> q = db[['FP89']]
 
     The different regimes in this data are given according to the North and
     South dummy (SOUTH).
 
-    >>> r_var = 'SOUTH'
-    >>> regimes = db.by_col(r_var)
+    >>> r_var = db['SOUTH']
 
     Since we want to perform tests for spatial dependence, we need to specify
     the spatial weights matrix that includes the spatial configuration of the
     observations into the error component of the model. To do that, we can open
     an already existing gal file or create a new one. In this case, we will
-    create one from ``NAT.shp``.
+    create one from our geopandas dataframe.
 
-    >>> w = Rook.from_shapefile(nat.get_path("natregimes.shp"))
+    >>> w = Rook.from_dataframe(db, use_index=True)
 
     Unless there is a good reason not to do it, the weights have to be
     row-standardized so every row of the matrix sums to one. Among other
@@ -263,77 +252,62 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
     parameters:
 
     >>> from spreg import TSLS_Regimes
-    >>> tslsr = TSLS_Regimes(y, x, yd, q, regimes, w=w, constant_regi='many', spat_diag=False, name_y=y_var, name_x=x_var, name_yend=yd_var, name_q=q_var, name_regimes=r_var, name_ds='NAT', name_w='NAT.shp')
+    >>> tslsr = TSLS_Regimes(y, x, yd, q, r_var, w=w, spat_diag=True, name_regimes='SOUTH', name_ds='NAT', name_w='NAT.shp')
 
     >>> tslsr.betas
     array([[ 3.66973562],
            [ 1.06950466],
            [ 0.14680946],
-           [ 2.45864196],
            [ 9.55873243],
            [ 1.94666348],
            [-0.30810214],
+           [ 2.45864196],
            [ 3.68718119]])
 
     >>> np.sqrt(tslsr.vm.diagonal())
-    array([0.38389901, 0.09963973, 0.04672091, 0.22725012, 0.49181223,
-           0.19630774, 0.07784587, 0.25529011])
+    array([0.46522151, 0.12074672, 0.05661795, 0.41893265, 0.16721773,
+           0.06631022, 0.27538921, 0.21745974])
 
     >>> print(tslsr.summary)
     REGRESSION RESULTS
     ------------------
     <BLANKLINE>
-    SUMMARY OF OUTPUT: TWO STAGE LEAST SQUARES ESTIMATION - REGIME 0
-    ----------------------------------------------------------------
+    SUMMARY OF OUTPUT: TWO STAGE LEAST SQUARES - REGIMES
+    ----------------------------------------------------
     Data set            :         NAT
     Weights matrix      :     NAT.shp
-    Dependent Variable  :      0_HR90                Number of Observations:        1673
-    Mean dependent var  :      3.3416                Number of Variables   :           4
-    S.D. dependent var  :      4.6795                Degrees of Freedom    :        1669
-    Pseudo R-squared    :      0.2092
+    Dependent Variable  :        HR90                Number of Observations:        3085
+    Mean dependent var  :      6.1829                Number of Variables   :           8
+    S.D. dependent var  :      6.6414                Degrees of Freedom    :        3077
+    Pseudo R-squared    :      0.4246
     <BLANKLINE>
     ------------------------------------------------------------------------------------
                 Variable     Coefficient       Std.Error     z-Statistic     Probability
     ------------------------------------------------------------------------------------
-              0_CONSTANT       3.6697356       0.3838990       9.5591172       0.0000000
-                  0_PS90       1.0695047       0.0996397      10.7337170       0.0000000
-                  0_UE90       0.1468095       0.0467209       3.1422643       0.0016765
-                  0_RD90       2.4586420       0.2272501      10.8191009       0.0000000
+              0_CONSTANT         3.66974         0.46522         7.88815         0.00000
+                  0_PS90         1.06950         0.12075         8.85742         0.00000
+                  0_UE90         0.14681         0.05662         2.59298         0.00951
+                  0_RD90         2.45864         0.27539         8.92788         0.00000
+              1_CONSTANT         9.55873         0.41893        22.81687         0.00000
+                  1_PS90         1.94666         0.16722        11.64149         0.00000
+                  1_UE90        -0.30810         0.06631        -4.64637         0.00000
+                  1_RD90         3.68718         0.21746        16.95570         0.00000
     ------------------------------------------------------------------------------------
-    Instrumented: 0_RD90
-    Instruments: 0_FP89
+    Instrumented: 0_RD90, 1_RD90
+    Instruments: 0_FP89, 1_FP89
     Regimes variable: SOUTH
     <BLANKLINE>
-    SUMMARY OF OUTPUT: TWO STAGE LEAST SQUARES ESTIMATION - REGIME 1
-    ----------------------------------------------------------------
-    Data set            :         NAT
-    Weights matrix      :     NAT.shp
-    Dependent Variable  :      1_HR90                Number of Observations:        1412
-    Mean dependent var  :      9.5493                Number of Variables   :           4
-    S.D. dependent var  :      7.0389                Degrees of Freedom    :        1408
-    Pseudo R-squared    :      0.2987
-    <BLANKLINE>
-    ------------------------------------------------------------------------------------
-                Variable     Coefficient       Std.Error     z-Statistic     Probability
-    ------------------------------------------------------------------------------------
-              1_CONSTANT       9.5587324       0.4918122      19.4357356       0.0000000
-                  1_PS90       1.9466635       0.1963077       9.9163867       0.0000000
-                  1_UE90      -0.3081021       0.0778459      -3.9578483       0.0000756
-                  1_RD90       3.6871812       0.2552901      14.4431026       0.0000000
-    ------------------------------------------------------------------------------------
-    Instrumented: 1_RD90
-    Instruments: 1_FP89
-    Regimes variable: SOUTH
-    ------------------------------------------------------------------------------------
-    GLOBAL DIAGNOSTICS
-    <BLANKLINE>
     REGIMES DIAGNOSTICS - CHOW TEST
                      VARIABLE        DF        VALUE           PROB
-                     CONSTANT         1          89.093           0.0000
-                         PS90         1          15.876           0.0001
-                         UE90         1          25.106           0.0000
-                         RD90         1          12.920           0.0003
-                  Global test         4         201.237           0.0000
+                CONSTANT              1         88.485           0.0000
+                    PS90              1         18.086           0.0000
+                    UE90              1         27.220           0.0000
+                    RD90              1         12.258           0.0005
+             Global test              4        195.994           0.0000
+    <BLANKLINE>
+    DIAGNOSTICS FOR SPATIAL DEPENDENCE
+    TEST                              DF         VALUE           PROB
+    Anselin-Kelejian Test             1        104.255           0.0000
     ================================ END OF REPORT =====================================
     """
 
@@ -348,12 +322,13 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
         robust=None,
         gwk=None,
         slx_lags=0,
+        slx_vars="all",
         sig2n_k=True,
         spat_diag=False,
         vm=False,
         constant_regi="many",
         cols2regi="all",
-        regime_err_sep=True,
+        regime_err_sep=False,
         name_y=None,
         name_x=None,
         cores=False,
@@ -368,8 +343,9 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
     ):
 
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
+        y, name_y = USER.check_y(y, n, name_y)
         USER.check_robust(robust, gwk)
+        yend, q, name_yend, name_q = USER.check_endog([yend, q], [name_yend, name_q])
         if robust == "hac":
             if regime_err_sep:
                 set_warn(
@@ -377,26 +353,21 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
                     "Error by regimes is not available for HAC estimation. The error by regimes has been disabled for this model.",
                 )
                 regime_err_sep = False
-            if spat_diag:
-                set_warn(
-                    self,
-                    "Spatial diagnostics are not available for HAC estimation. The spatial diagnostics have been disabled for this model.",
-                )
-                spat_diag = False
-        USER.check_spat_diag(spat_diag, w)
+        spat_diag, warn = USER.check_spat_diag(spat_diag=spat_diag, w=w, robust=robust, slx_lags=slx_lags)
+        set_warn(self, warn)
         x_constant, name_x, warn = USER.check_constant(x, name_x, just_rem=True)
         set_warn(self, warn)
         name_x = USER.set_name_x(name_x, x_constant, constant=True)
+        w = USER.check_weights(w, y, slx_lags=slx_lags, w_required=spat_diag)
         if slx_lags > 0:
-            USER.check_weights(w, y, w_required=True)
-            lag_x = get_lags(w, x_constant, slx_lags)
-            x_constant = np.hstack((x_constant, lag_x))
-            name_x += USER.set_name_spatial_lags(name_x, slx_lags)
-        else:
-            USER.check_weights(w, y, w_required=False)
+            x_constant,name_x = USER.flex_wx(w,x=x_constant,name_x=name_x,constant=False,
+                                                slx_lags=slx_lags,slx_vars=slx_vars)
+            set_warn(self,"WX is computed using the complete W, i.e. not trimmed by regimes.")
+
         self.constant_regi = constant_regi
         self.cols2regi = cols2regi
         self.name_ds = USER.set_name_ds(name_ds)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
         self.name_regimes = USER.set_name_ds(name_regimes)
         self.name_w = USER.set_name_w(name_w, w)
         self.name_gwk = USER.set_name_w(name_gwk, gwk)
@@ -441,10 +412,10 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
                 latex
             )
         else:
-            q, self.name_q = REGI.Regimes_Frame.__init__(
+            q, self.name_q, xtype = REGI.Regimes_Frame.__init__(
                 self, q, regimes, constant_regi=None, cols2regi="all", names=name_q
             )
-            x, self.name_x, x_rlist = REGI.Regimes_Frame.__init__(
+            x, self.name_x, xtype, x_rlist = REGI.Regimes_Frame.__init__(
                 self,
                 x_constant,
                 regimes,
@@ -453,7 +424,7 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
                 names=name_x,
                 rlist=True
             )
-            yend, self.name_yend, yend_rlist = REGI.Regimes_Frame.__init__(
+            yend, self.name_yend, xtypeyd, yend_rlist = REGI.Regimes_Frame.__init__(
                 self,
                 yend,
                 regimes,
@@ -465,7 +436,7 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
             )
             self.output = pd.DataFrame(self.name_x+self.name_yend,
                                        columns=['var_names'])
-            self.output['var_type'] = ['x']*len(self.name_x)+['yend']*len(self.name_yend)
+            self.output['var_type'] = xtype+xtypeyd
             self.output['regime'] = x_rlist+yend_rlist
             self.output['equation'] = 0
 
@@ -633,7 +604,7 @@ class TSLS_Regimes(BaseTSLS, REGI.Regimes_Frame):
             self.name_z += results[r].name_z
             self.name_h += results[r].name_h
             self.output = pd.concat([self.output, pd.DataFrame({'var_names': results[r].name_x+results[r].name_yend,
-                                                       'var_type': ['x']*len(results[r].name_x)+['yend']*len(results[r].name_yend),
+                                                       'var_type': ['o']+['x']*(len(results[r].name_x)-1)+['yend']*len(results[r].name_yend),
                                                        'regime': r, 'equation': r})], ignore_index=True)
 
             counter += 1
@@ -753,6 +724,385 @@ def _optimal_weight(reg, sig2n_k, warn=True):
         )
     return
 
+class TSLS_Endog_Regimes(TSLS_Regimes):
+
+    """
+    Two stage least squares (S2SLS) with endogenous regimes. 
+    Based on the function skater_reg as shown in :cite:`Anselin2021`.
+
+    Parameters
+    ----------
+    y            : numpy.ndarray or pandas.Series
+                   nx1 array for dependent variable
+    x            : numpy.ndarray or pandas object
+                   Two dimensional array with n rows and one column for each
+                   independent (exogenous) variable, excluding the constant
+    yend         : numpy.ndarray or pandas object
+                   Two dimensional array with n rows and one column for each
+                   endogenous variable
+    q            : numpy.ndarray or pandas object
+                   Two dimensional array with n rows and one column for each
+                   external exogenous variable to use as instruments (note:
+                   this should not contain any variables from x)
+    w            : pysal W object
+                   Spatial weights object (required if running spatial
+                   diagnostics)
+    n_clusters   : int
+                   Number of clusters to be used in the endogenous regimes.
+                   If None (default), the number of clusters will be chosen
+                   according to the function utils.optim_k using a method 
+                   adapted from Mojena (1977)'s Rule Two
+    quorum       : int
+                   Minimum number of observations in a cluster to be considered
+                   Must be at least larger than the number of variables in x
+                   Default value is 30 or 10*k, whichever is larger.
+    trace        : boolean
+                   Sets  whether to store intermediate results of the clustering
+                   Hard-coded to True if n_clusters is None
+    name_y       : string
+                   Name of dependent variable for use in output
+    name_x       : list of strings
+                   Names of independent variables for use in output
+    name_yend    : list of strings
+                   Names of endogenous variables for use in output
+    name_q       : list of strings
+                   Names of instruments for use in output
+    name_w       : string
+                   Name of weights matrix for use in output
+    name_gwk     : string
+                   Name of kernel weights matrix for use in output
+    name_ds      : string
+                   Name of dataset for use in output
+    name_regimes : string
+                   Name of regimes variable for use in output
+    latex        : boolean
+                   Specifies if summary is to be printed in latex format
+    **kwargs     : additional keyword arguments depending on the specific model
+
+    Attributes
+    ----------
+    output       : dataframe
+                   regression results pandas dataframe
+    summary      : string
+                   Summary of regression results and diagnostics (note: use in
+                   conjunction with the print command)
+    betas        : array
+                   kx1 array of estimated coefficients
+    u            : array
+                   nx1 array of residuals
+    e_pred       : array
+                   nx1 array of residuals (using reduced form)
+    predy        : array
+                   nx1 array of predicted y values
+    predy_e      : array
+                   nx1 array of predicted y values (using reduced form)
+    n            : integer
+                   Number of observations
+    k            : integer
+                   Number of variables for which coefficients are estimated
+                   (including the constant)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    kstar        : integer
+                   Number of endogenous variables.
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    y            : array
+                   nx1 array for dependent variable
+    x            : array
+                   Two dimensional array with n rows and one column for each
+                   independent (exogenous) variable, including the constant
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    yend         : array
+                   Two dimensional array with n rows and one column for each
+                   endogenous variable
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    q            : array
+                   Two dimensional array with n rows and one column for each
+                   external exogenous variable used as instruments
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    z            : array
+                   nxk array of variables (combination of x and yend)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    h            : array
+                   nxl array of instruments (combination of x and q)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    robust       : string
+                   Adjustment for robust standard errors
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    mean_y       : float
+                   Mean of dependent variable
+    std_y        : float
+                   Standard deviation of dependent variable
+    vm           : array
+                   Variance covariance matrix (kxk)
+    pr2          : float
+                   Pseudo R squared (squared correlation between y and ypred)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    pr2_e        : float
+                   Pseudo R squared (squared correlation between y and ypred_e
+                   (using reduced form))
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    utu          : float
+                   Sum of squared residuals
+    sig2         : float
+                   Sigma squared used in computations
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    std_err      : array
+                   1xk array of standard errors of the betas
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    z_stat       : list of tuples
+                   z statistic; each tuple contains the pair (statistic,
+                   p-value), where each is a float
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    ak_test      : tuple
+                   Anselin-Kelejian test; tuple contains the pair (statistic,
+                   p-value)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    cfh_test     : tuple
+                   Common Factor Hypothesis test; tuple contains the pair (statistic,
+                   p-value). Only when it applies (see specific documentation).
+    name_y       : string
+                   Name of dependent variable for use in output
+    name_x       : list of strings
+                   Names of independent variables for use in output
+    name_yend    : list of strings
+                   Names of endogenous variables for use in output
+    name_z       : list of strings
+                   Names of exogenous and endogenous variables for use in
+                   output
+    name_q       : list of strings
+                   Names of external instruments
+    name_h       : list of strings
+                   Names of all instruments used in ouput
+    name_w       : string
+                   Name of weights matrix for use in output
+    name_gwk     : string
+                   Name of kernel weights matrix for use in output
+    name_ds      : string
+                   Name of dataset for use in output
+    name_regimes : string
+                   Name of regimes variable for use in output
+    title        : string
+                   Name of the regression method used
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    sig2n        : float
+                   Sigma squared (computed with n in the denominator)
+    sig2n_k      : float
+                   Sigma squared (computed with n-k in the denominator)
+    hth          : float
+                   :math:`H'H`.
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    hthi         : float
+                   :math:`(H'H)^{-1}`.
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    varb         : array
+                   :math:`(Z'H (H'H)^{-1} H'Z)^{-1}`.
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    zthhthi      : array
+                   :math:`Z'H(H'H)^{-1}`.
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    pfora1a2     : array
+                   n(zthhthi)'varb
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    sp_multipliers: dict
+                   Dictionary of spatial multipliers (if spat_impacts is not None)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    regimes      : list
+                   List of n values with the mapping of each
+                   observation to a regime. Assumed to be aligned with 'x'.
+    constant_regi: string
+                   Ignored if regimes=False. Constant option for regimes.
+                   Switcher controlling the constant term setup. It may take
+                   the following values:
+
+                   * 'one': a vector of ones is appended to x and held constant across regimes.
+
+                   * 'many': a vector of ones is appended to x and considered different per regime.
+    cols2regi    : list, 'all'
+                   Ignored if regimes=False. Argument indicating whether each
+                   column of x should be considered as different per regime
+                   or held constant across regimes (False).
+                   If a list, k booleans indicating for each variable the
+                   option (True if one per regime, False to be held constant).
+                   If 'all', all the variables vary by regime.
+    regime_lag_sep: boolean
+                    If True, the spatial parameter for spatial lag is also
+                    computed according to different regimes. If False (default),
+                    the spatial parameter is fixed accross regimes.
+    regime_err_sep: boolean
+                    If True, a separate regression is run for each regime.
+    kr           : int
+                   Number of variables/columns to be "regimized" or subject
+                   to change by regime. These will result in one parameter
+                   estimate by regime for each variable (i.e. nr parameters per
+                   variable)
+    kf           : int
+                   Number of variables/columns to be considered fixed or
+                   global across regimes and hence only obtain one parameter
+                   estimate
+    nr           : int
+                   Number of different regimes in the 'regimes' list
+    multi        : dictionary
+                   Only available when multiple regressions are estimated,
+                   i.e. when regime_err_sep=True and no variable is fixed
+                   across regimes.
+                   Contains all attributes of each individual regression
+    SSR          : list
+                   list with the total sum of squared residuals for the model 
+                   considering all regimes for each of steps of number of regimes
+                   considered, starting with the solution with 2 regimes. 
+    clusters     : int
+                   Number of clusters considered in the endogenous regimes
+    _trace       : list
+                   List of dictionaries with the clustering results for each
+                   number of clusters tested. Only available if n_clusters is
+                   None or trace=True.
+    Examples
+    --------
+    >>> import libpysal
+    >>> import numpy as np
+    >>> np.set_printoptions(legacy='1.25') #to avoid printing issues with numpy floats
+    >>> import geopandas as gpd
+    >>> from spreg import TSLS_Endog_Regimes
+
+    Open data on Baltimore house sales price and characteristics in Baltimore
+    from libpysal examples using geopandas.
+
+    >>> db = gpd.read_file(libpysal.examples.get_path('baltim.shp'))
+
+    We will create a weights matrix based on contiguity.
+
+    >>> w = libpysal.weights.Queen.from_dataframe(db, use_index=True)
+    >>> w.transform = "r"
+
+    For this example, just to show how the function works, we will use 
+    the 'PRICE' column as the dependent variable, 'AGE' as independent variable, 
+    'SQFT' as endogenous variable, and 'NROOM' as an instrument.
+    At this point, we will let the model choose the number of clusters.
+
+    >>> reg = TSLS_Endog_Regimes(y=db['PRICE'], x=db['AGE'], yend=db['SQFT'], q=db['NROOM'], w=w)
+    
+    The function `print(reg.summary)` can be used to visualize the results of the regression.
+
+    Alternatively, we can check individual attributes:
+    >>> reg.betas
+    array([[28.55593008],
+           [-0.27351083],
+           [22.36160147],
+           [-0.64768173],
+           [22.89318454],
+           [-0.48438473],
+           [ 1.11252432],
+           [ 2.74757415],
+           [ 2.26013585]])
+    >>> reg.SSR
+    [77331.25758888898, 74978.74262486391, 73326.08720216743]
+    >>> reg.clusters
+    array([0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0,
+           0, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 2, 0, 2, 2,
+           2, 2, 2, 2, 2, 2, 0, 2, 1, 0, 1, 0, 0, 2, 0, 1, 0, 0, 0, 0, 1, 0,
+           1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0, 2, 0, 0, 0, 2, 2,
+           2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 2, 0, 2, 0, 2, 2, 2, 0, 0, 0, 0,
+           0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], dtype=int32)
+
+    We  will now set the number of clusters to 2 and run the regression again.
+
+    >>> reg = TSLS_Endog_Regimes(y=db['PRICE'], x=db['AGE'], yend=db['SQFT'], q=db['NROOM'], w=w, n_clusters=2)
+
+    The function `print(reg.summary)` can be used to visualize the results of the regression.
+
+    Alternatively, we can check individual attributes as before:
+    >>> reg.betas
+    array([[29.89584104],
+           [-0.36936638],
+           [22.36160147],
+           [-0.64768173],
+           [ 1.41273696],
+           [ 2.74757415]])
+    >>> reg.SSR
+    [77331.25758888898]
+    >>> reg.clusters
+    array([0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0,
+           0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0,
+           1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], dtype=int32)
+    """
+
+
+    def __init__(
+        self, y, x, yend, q, w=None, n_clusters=None, quorum=-1, trace=True, name_y=None, name_x=None, name_yend=None, name_q=None, 
+        constant_regi='many', cols2regi='all', regime_err_sep=True, **kwargs):
+
+        n = USER.check_arrays(y, x)
+        y, name_y = USER.check_y(y, n, name_y)
+        yend, q, name_yend, name_q = USER.check_endog([yend, q], [name_yend, name_q])
+        w = USER.check_weights(w, y, w_required=True)
+        x_constant, name_x, warn = USER.check_constant(x, name_x, just_rem=True)
+        set_warn(self, warn)
+        warn = USER.check_regi_args(constant_regi, cols2regi, regime_err_sep)
+        set_warn(self, warn)
+        # Standardize the variables
+        x_combined = np.hstack((x_constant, yend))
+        x_std = (x_combined - np.mean(x_combined, axis=0)) / np.std(x_combined, axis=0)
+
+        if quorum < 0:
+            quorum = np.max([(x_constant.shape[1]+1)*10, 30])
+        
+        if not n_clusters:
+            n_clusters_opt = x_constant.shape[0]*0.70//quorum
+            if n_clusters_opt < 2:
+                raise ValueError(
+                    "The combination of the values of `N` and `quorum` is not compatible with regimes estimation.")
+            sk_reg_results = Skater_reg().fit(n_clusters_opt, w, x_std, {'reg':BaseTSLS,'y':y,'x':x_constant,'yend':yend,'q':q}, quorum=quorum, trace=True)
+            n_clusters = optim_k([sk_reg_results._trace[i][1][2] for i in range(1, len(sk_reg_results._trace))])
+            self.clusters = sk_reg_results._trace[n_clusters-1][0]
+            self.score = sk_reg_results._trace[n_clusters-1][1][2]
+        else:
+            try:
+                sk_reg_results = Skater_reg().fit(n_clusters, w, x_std, {'reg':BaseTSLS,'y':y,'x':x_constant,'yend':yend,'q':q}, quorum=quorum, trace=trace)
+                self.clusters = sk_reg_results.current_labels_
+                self.score = sk_reg_results._trace[-1][1][2]
+            except Exception as e:
+                if str(e) == "one or more input arrays have more columns than rows":
+                    raise ValueError("One or more input ended up with more variables than observations. Please check your setting for `quorum`.")
+                else:
+                    print("An error occurred:", e)
+
+        self._trace = sk_reg_results._trace
+        self.SSR = [self._trace[i][1][2] for i in range(1, len(self._trace))]
+        TSLS_Regimes.__init__(self, y, x_constant, yend, q, regimes=self.clusters, w=w, name_y=name_y, name_x=name_x, name_yend=name_yend, 
+                              name_q=name_q, name_regimes='Skater_reg', constant_regi='many', cols2regi='all', regime_err_sep=True, **kwargs)
+
 
 def _test():
     import doctest
